# Generated by Django 5.2.4 on 2025-10-05 10:41

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0008_profile_ai_create_attack_navigator_layer'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='include_embedded_relationships_attributes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=128), default=list, size=None),
        ),
    ]
