# Generated by Django 5.1.5 on 2025-06-24 12:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0004_profile_identity_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='generate_pdf',
            field=models.BooleanField(default=False),
        ),
    ]
