# Generated by Django 5.1.5 on 2025-07-07 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0005_profile_generate_pdf'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='ai_create_attack_flow',
            field=models.BooleanField(default=False),
        ),
    ]
