# Generated by Django 5.2.4 on 2025-07-14 15:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dogesec_stixifier', '0007_remove_profile_ai_summary_provider_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='profile',
            name='ai_create_attack_navigator_layer',
            field=models.BooleanField(default=False),
        ),
    ]
